#ifndef FLE_ListView_H
#define FLE_ListView_H

#include "FLE_List.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <fltk/Fl.h>
#include <fltk/Fl_Group.h>
#include <fltk/Fl_Slider.h>
#include <fltk/Fl_Scrollbar.h>
#include <fltk/Fl_Image.h>

#include <fltk/fl_draw.h>
#include <fltk/x.h>

class FLE_ListHeader : public Fl_Widget
{
public:
	FLE_ListHeader(int X,int Y,int W,int H,const char*l=0) : Fl_Widget(X,Y,W,H,l) {
		cols=0;
		for(int a=0;a<10;a++) { colw[a]=0; colf[a]=FL_ALIGN_LEFT; coli[a]=0;}
    box(FL_THIN_UP_BOX);
    color(FL_GRAY);
	}

	void add_column(const char *name, int w) {
		++cols;	
		colw[cols-1] = w;
		coln[cols-1] = strdup(name);
	}

	int columns() { return cols; }
	int column_width(int c) { return colw[c]; }
	void column_width(int c, int w) { colw[c] = w; }
	
	const char *column_name(int c) { return coln[c]; }
	void column_name(int c, const char *name) { if(coln[c]) delete []coln[c]; coln[c] = strdup(name); redraw(); }
	
	int column_flags(int c) { return colf[c]; }
	void column_flags(int c, int f) { colf[c] = f; }

	Fl_Image *column_image(int c) { return coli[c]; }
	void column_image(int c, Fl_Image *im) { coli[c] = im; }

	void draw();
	void layout();
	int handle(int ev);

	
	int cols;
	int colw[10]; //widths
	int colf[10]; //drawing flags
	char *coln[10]; //labels
	Fl_Image *coli[10]; //images
};

class FLE_ListView : public Fl_Group {
public:	
	FLE_ListView(int X,int Y,int W,int H,const char*l=0);
	~FLE_ListView();
	
	FLE_ListHeader *header() { return _header; }
	void header(FLE_ListHeader *h) { _header = h; }

	void add_column(const char *name, int w=-1) { _header->add_column(name, w); if(w<0) find_def=true; }
	int columns() { return _header->columns(); }
	int column_width(int c) { return _header->column_width(c); }
	void column_width(int c, int w) { _header->column_width(c, w); if(w<0) find_def=true; }
	const char *column_name(int c) { return _header->column_name(c); }
	void column_name(int c, const char *name) { _header->column_name(c, name); }
	int column_flags(int c) { return _header->column_flags(c); }
	void column_flags(int c, int f) { _header->column_flags(c,f); }
	Fl_Image *column_image(int c) { return _header->column_image(c); }
	void column_image(int c, Fl_Image *im) { _header->column_image(c,im); }
	
	int move()  { return (type() & MOVE_SELECTION);  }
	int multi() { return (type() & MULTI_SELECTION); }
	int single(){ return !(type() & MULTI_SELECTION);}

	int totalheight() { return total_height; }	
	
	int	yposition() const {return yposition_;}
	void yposition(int Y);
	int	xposition() const {return xposition_;}
	void xposition(int X);
	
	Fl_Widget* top(); 						// move to top and set first item to current
	Fl_Widget* item_at(int Y);    // Return item at Y

	Fl_Widget *item(Fl_Widget *i) { return (item_ = i); }
	Fl_Widget *item() { return item_; }
	void show_item(Fl_Widget *w);

	Fl_Widget* next();
	Fl_Widget* prev();

	void damage_item(Fl_Widget *i); // make this item redraw

	void scroll_up(int pixels = 20);
	void scroll_down(int pixels = 20);

	void clear();
	void remove(int index);
	void remove(Fl_Widget& o) { int w = Fl_Group::find(o);  remove(w);}
	void remove(Fl_Widget* o) { int w = Fl_Group::find(*o); remove(w);}

	void sort_selection(); //Call this always before when new multiselection
	void moveselection_up(int dy);
	void moveselection_down(int dy);

	void clear_selection();  
	void remove_selection();
		
	int select(Fl_Widget *w, int value = 1);
	int select_only(Fl_Widget *w);
	void select_items(int from, int to);

	int unselect_all();
	int selected() { return selection.count(); }

 	enum { // values for type()
		HORIZONTAL = 1,
		VERTICAL = 2,
		BOTH = 3,
		ALWAYS_ON = 4,
		HORIZONTAL_ALWAYS = 5,
		VERTICAL_ALWAYS = 6,
		BOTH_ALWAYS = 7,
		
		MULTI_SELECTION = 8,
		MOVE_SELECTION  = 16
	};

// Virtual functions
	virtual int handle(int ev);
	virtual void layout();
	virtual void layout_scrollbars();
  
	virtual void draw();	
	virtual void draw_header();	
	virtual void draw_row(int x, int y, int w, int h, int selected);

	FLE_List<Fl_Widget> &get_selection() { return selection; }

	Fl_Scrollbar vscrollbar;
	Fl_Scrollbar hscrollbar;

private:
	static void hscrollbar_cb(Fl_Widget*, void*);
	static void vscrollbar_cb(Fl_Widget*, void*);

	bool find_def; // Set when needs to find default colmn sizes (called by layout)
	void find_def_sizes();

	FLE_ListHeader head; //Default header
	FLE_ListHeader *_header; //Pointer to current header

	Fl_Widget *item_;      // Current item	

	int total_height;
	int yposition_, xposition_;
	int scrolldy, scrolldx;

	void draw_item(Fl_Widget *);
	void draw_clip(int,int,int,int);
	static void draw_clip_cb(void*,int,int,int,int);

	FLE_List<Fl_Widget> selection;
	bool on_selection(int Y);

	// number of first visible item
	int first_vis;

	int X,Y,W,H; //box
	int HX,HY,HW,HH; //header box
};

#endif

